# Arch Perl library, Copyright (C) 2004 Mikhael Goikhman
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use 5.005;
use strict;

package Arch::Log;

sub new ($$) {
	my $class = shift;
	my $message = shift || die "Arch::Log::new: no message\n";

	my $self = {
		message => $message,
		headers => undef,
	};

	return bless $self, $class;
}

# non-method utility function
sub group_array ($$) {
	my @array = @{shift()};
	my $num_items = shift;
	my @new_array = ();
	push @new_array, [ splice @array, 0, $num_items ] while @array;
	return @new_array if wantarray;
	return \@new_array;
}

# non-method utility function
sub parse_message ($;$) {
	my $message = shift;
	my $list_headers = shift || {};
	my ($headers_str, $body) = $message =~ /^(.*?\n)\n(.*)$/s
		or die "Incorrect message:\n\n$message\n\n- No body delimeter\n";

	my $headers = { body => $body };
	$headers_str =~ s{^([\w-]+):[ \t]*(.*\n(?:[ \t]+.*\n)*)}{
		my ($header, $value) = (lc($1), $2);
		$header =~ s/-/_/sg;
		chomp($value);
		$value = [ split(/[ \n]+/, $value) ] if $list_headers->{$header};
		die "Duplicate header $header in message:\n\n$message\n"
			if exists $headers->{$header};
		$headers->{$header} = $value;
		""
	}meg;

	#print "*** $_: $headers->{$_} ***\n" foreach keys %$headers;
	return $headers;
}

sub get_message ($) {
	my $self = shift;
	return $self->{message};
}

sub get_headers ($) {
	my $self = shift;
	return $self->{headers} if defined $self->{headers};

	my $headers = parse_message($self->{message}, {
		modified_files => 1,
		renamed_files => 1,
		new_files => 1,
		removed_files => 1,
		renamed_directories => 1,
		new_directories => 1,
		removed_directories => 1,
		new_patches => 1,
	});

	$headers->{$_} &&= group_array($headers->{$_}, 2)
		foreach qw(renamed_files renamed_directories);

	return $self->{headers} = $headers;
}

1;
