# Arch Perl library, Copyright (C) 2004 Mikhael Goikhman
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use 5.005;
use strict;

package Arch::TempFiles;

#use File::Temp;
use Exporter;
use vars qw($temp_root @temp_files @temp_dirs @ISA @EXPORT_OK);
@ISA = qw(Exporter);
@EXPORT_OK = qw(temp_root temp_name temp_file_name temp_dir_name);

BEGIN {
	$temp_root = "/tmp";
	@temp_files = ();
	@temp_dirs = ();
}

END {
	# temporarily use system()
	system("/bin/rm -rf @temp_files @temp_dirs") if @temp_files + @temp_dirs;
}

sub temp_root (;$) {
	$temp_root = shift if @_;
	return $temp_root;
}

sub temp_name (;$) {
	my $name = shift || "arch";
	die "Can't make temporary $name name, no valid temp root defined\n"
		unless $temp_root && -d $temp_root;
	my $prefix = "$temp_root/,,$name-";
	my $file_name;
	my $tries = 10000;
	do {
		$file_name = $prefix . sprintf("%06d", rand(1000000));
	} while -e $file_name && --$tries;
	die "Failed to acquire unused temp name $prefix*\n" unless $tries;
	return $file_name;
}

sub temp_file_name (;$) {
	my $file_name = temp_name($_[0]);
	push @temp_dirs, $file_name;
	return $file_name;
}

sub temp_dir_name (;$) {
	my $dir_name = temp_name($_[0]);
	push @temp_dirs, $dir_name;
	return $dir_name;
}

1;
